@@ -29,9 +29,9 @@ class LibvirtNFSVolumeDriverTestCase(test_volume.LibvirtVolumeBaseTestCase):
         self.mnt_base = '/mnt'
         self.flags(nfs_mount_point_base=self.mnt_base, group='libvirt')
 
-    def test_libvirt_nfs_driver(self):
-        libvirt_driver = nfs.LibvirtNFSVolumeDriver(self.fake_conn)
-        self.stubs.Set(libvirt_utils, 'is_mounted', lambda x, d: False)
+    @mock.patch.object(libvirt_utils, 'is_mounted', return_value=False)
+    def test_libvirt_nfs_driver(self, mock_is_mounted):
+        libvirt_driver = nfs.LibvirtNFSVolumeDriver(self.fake_host)
 
         export_string = '192.168.1.1:/nfs/share1'
         export_mnt_base = os.path.join(self.mnt_base,
@@ -50,6 +50,7 @@ class LibvirtNFSVolumeDriverTestCase(test_volume.LibvirtVolumeBaseTestCase):
             ('mount', '-t', 'nfs', export_string, export_mnt_base),
             ('umount', export_mnt_base)]
         self.assertEqual(expected_commands, self.executes)
+        self.assertTrue(mock_is_mounted.called)
 
     @mock.patch.object(nfs.utils, 'execute')
     @mock.patch.object(nfs.LOG, 'debug')
@@ -59,7 +60,7 @@ class LibvirtNFSVolumeDriverTestCase(test_volume.LibvirtVolumeBaseTestCase):
         export_string = '192.168.1.1:/nfs/share1'
         connection_info = {'data': {'export': export_string,
                                     'name': self.name}}
-        libvirt_driver = nfs.LibvirtNFSVolumeDriver(self.fake_conn)
+        libvirt_driver = nfs.LibvirtNFSVolumeDriver(self.fake_host)
         mock_utils_exe.side_effect = processutils.ProcessExecutionError(
             None, None, None, 'umount', 'umount: device is busy.')
         libvirt_driver.disconnect_volume(connection_info, "vde")
@@ -78,7 +79,7 @@ class LibvirtNFSVolumeDriverTestCase(test_volume.LibvirtVolumeBaseTestCase):
         self.assertTrue(mock_LOG_exception.called)
 
     def test_libvirt_nfs_driver_get_config(self):
-        libvirt_driver = nfs.LibvirtNFSVolumeDriver(self.fake_conn)
+        libvirt_driver = nfs.LibvirtNFSVolumeDriver(self.fake_host)
         export_string = '192.168.1.1:/nfs/share1'
         export_mnt_base = os.path.join(self.mnt_base,
                                        utils.get_hash_str(export_string))
@@ -93,8 +94,9 @@ class LibvirtNFSVolumeDriverTestCase(test_volume.LibvirtVolumeBaseTestCase):
         self.assertEqual('raw', tree.find('./driver').get('type'))
         self.assertEqual('native', tree.find('./driver').get('io'))
 
-    def test_libvirt_nfs_driver_already_mounted(self):
-        libvirt_driver = nfs.LibvirtNFSVolumeDriver(self.fake_conn)
+    @mock.patch.object(libvirt_utils, 'is_mounted', return_value=True)
+    def test_libvirt_nfs_driver_already_mounted(self, mock_is_mounted):
+        libvirt_driver = nfs.LibvirtNFSVolumeDriver(self.fake_host)
 
         export_string = '192.168.1.1:/nfs/share1'
         export_mnt_base = os.path.join(self.mnt_base,
@@ -106,14 +108,12 @@ class LibvirtNFSVolumeDriverTestCase(test_volume.LibvirtVolumeBaseTestCase):
         libvirt_driver.disconnect_volume(connection_info, "vde")
 
         expected_commands = [
-            ('findmnt', '--target', export_mnt_base, '--source',
-             export_string),
             ('umount', export_mnt_base)]
         self.assertEqual(expected_commands, self.executes)
 
-    def test_libvirt_nfs_driver_with_opts(self):
-        libvirt_driver = nfs.LibvirtNFSVolumeDriver(self.fake_conn)
-        self.stubs.Set(libvirt_utils, 'is_mounted', lambda x, d: False)
+    @mock.patch.object(libvirt_utils, 'is_mounted', return_value=False)
+    def test_libvirt_nfs_driver_with_opts(self, mock_is_mounted):
+        libvirt_driver = nfs.LibvirtNFSVolumeDriver(self.fake_host)
         export_string = '192.168.1.1:/nfs/share1'
         options = '-o intr,nfsvers=3'
         export_mnt_base = os.path.join(self.mnt_base,
@@ -132,3 +132,4 @@ class LibvirtNFSVolumeDriverTestCase(test_volume.LibvirtVolumeBaseTestCase):
             ('umount', export_mnt_base),
         ]
         self.assertEqual(expected_commands, self.executes)
+        self.assertTrue(mock_is_mounted.called)
