@@ -824,6 +824,9 @@ class LibvirtConfigGuestDisk(LibvirtConfigGuestDevice):
         if self.boot_order:
             dev.append(etree.Element("boot", order=self.boot_order))
 
+        if self.device_addr:
+            dev.append(self.device_addr.format_dom())
+
         return dev
 
     def parse_dom(self, xmldoc):
@@ -1138,6 +1141,11 @@ class LibvirtConfigGuestDeviceAddress(LibvirtConfigObject):
             root_name='address', **kwargs)
         self.type = type
 
+    def format_dom(self):
+        xml = super(LibvirtConfigGuestDeviceAddress, self).format_dom()
+        xml.set("type", self.type)
+        return xml
+
     @staticmethod
     def parse_dom(xmldoc):
         addr_type = xmldoc.get('type')
@@ -1160,6 +1168,20 @@ class LibvirtConfigGuestDeviceAddressDrive(LibvirtConfigGuestDeviceAddress):
         self.target = None
         self.unit = None
 
+    def format_dom(self):
+        xml = super(LibvirtConfigGuestDeviceAddressDrive, self).format_dom()
+
+        if self.controller is not None:
+            xml.set("controller", str(self.controller))
+        if self.bus is not None:
+            xml.set("bus", str(self.bus))
+        if self.target is not None:
+            xml.set("target", str(self.target))
+        if self.unit is not None:
+            xml.set("unit", str(self.unit))
+
+        return xml
+
     def parse_dom(self, xmldoc):
         self.controller = xmldoc.get('controller')
         self.bus = xmldoc.get('bus')
